#ifndef A48_H
#define A48_H

#include <cstdio>
#include <iostream>
#include <set>
#include <map>
#include <iterator>
#include "heap.h"
#include "vec3.h"
#include "vec4.h"
#include "rply.h"


namespace A48 {

typedef class Mesh;
typedef class Face;
typedef class Edge;
typedef class Hedge;
typedef class Vertex;

typedef class Markable;
typedef class Error;

typedef std::set<Face*>  FaceContainer; 
typedef FaceContainer::iterator FaceIter;

typedef std::set<Edge*>  EdgeContainer;
typedef EdgeContainer::iterator EdgeIter;

typedef std::set<Vertex*>  VertexContainer;
typedef VertexContainer::iterator VertexIter;

typedef class Ipair;
typedef std::map<const Ipair, A48::Hedge*> HedgeMap;

class Ipair {
  int i0; int i1;
public:
  Ipair(int p0, int p1) { i0=p0; i1=p1; }
  bool operator<(const Ipair s) const {
    return (i0 == s.i0)? (i1 < s.i1) : (i0 < s.i0);
  }
};

class Markable {
  bool     mark_;
 public:
  Markable(void) : mark_(false) {};
  void set_mark(bool m) { mark_ = m; };
  bool is_marked(void) const { return mark_; };
};


typedef Vec3 Vector3;

struct Point {
  Vector3     g;    // 3D position
  Point() {}
  Point(Vector3 pos) : g(pos) {}
  Point(double x, double y, double z) { g[X]=x; g[Y]=y; g[Z]=z; }
};


class Error {
 public:
  Error(char *s=""){ 
    std::cerr<<"AM error: "<<s<<std::endl; exit(0); 
  }
};

int ply_read_file(char *fname, Mesh *m);

}


#include "face.h"
#include "edge.h"
#include "vertex.h"
#include "mesh.h"


#endif
