#include "mywindow.h"
#include "Kdtree.h"
#include <stdio.h>
#include <stdlib.h>
#include <set>

#include <FL/Fl_Button.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Value_Slider.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Round_Button.H>


Fl_Double_Window *window;
mywindow *janelagl;
Fl_Button *btnOpenIter,*btnErro1,*btnErro2,*btnErro3,*btnErro4, *btnErro5,*btnEstratificar, *btnDesestratificar;
Fl_Value_Slider *sliderIteracao,*sliderError,*sliderDisk,*sliderThreshold;
Fl_Check_Button *cbMalhas,*cbRec;
Fl_Box *boxel,*boxgrupo;
Fl_Round_Button *rb1,*rb2;

bool isInTriangle(double x, double y, Face *f, double *depth, double *nx, double *ny, double *nz);
double areaTriangulo(double x1, double y1, double x2, double y2, double x3, double y3);
void criarDrawableReconstruction(Iteration *iteracao, Kdtree *kdt);
void calcularVizinhos();
void callback_estratificar(Fl_Widget *);

float reconstruction_threshold;
vector<Iteration *> iteracoes;
vector<Mesh *> scans;
int erroatual;
int metodo;
float estrat_param;
bool stratified;

gsl_matrix *calcularMatrizCovariancia(Mesh *malha)
{
	gsl_matrix *covmat = gsl_matrix_calloc (3,3);
	gsl_matrix *vec = gsl_matrix_calloc(3,1);
	float mx=0.;
	float my=0.;
	float mz=0.;
	for(VertexIter it=malha->verts_begin();it!=malha->verts_end();it++)
	{
		mx+=(*it)->a.g(0);	
		my+=(*it)->a.g(1);
		mz+=(*it)->a.g(2);
	}
	mx/=(float)malha->num_verts();
	my/=(float)malha->num_verts();
	mz/=(float)malha->num_verts();
	
	for(VertexIter it=malha->verts_begin();it!=malha->verts_end();it++)
	{
		gsl_matrix_set(vec,0,0,(*it)->a.g(0)-mx);
		gsl_matrix_set(vec,1,0,(*it)->a.g(1)-my);
		gsl_matrix_set(vec,2,0,(*it)->a.g(2)-mz);
		gsl_blas_dgemm(CblasNoTrans,CblasTrans,1.,vec,vec,1.,covmat);
	}
	gsl_matrix_scale(covmat,1./((float)malha->num_verts()-1));
	gsl_matrix_free(vec);
	return covmat;
}

gsl_matrix *calcularEixosPrincipais(gsl_matrix *covar)
{
	gsl_eigen_symmv_workspace *wspace = gsl_eigen_symmv_alloc(3);
	gsl_vector *autovalores = gsl_vector_alloc(3);
	gsl_matrix *autovetores = gsl_matrix_alloc(3,3);
	gsl_eigen_symmv(covar,autovalores,autovetores,wspace);
	gsl_eigen_symmv_free(wspace);
	gsl_eigen_symmv_sort(autovalores,autovetores,GSL_EIGEN_SORT_VAL_DESC);
	gsl_vector_free(autovalores);
	return autovetores;
}

void mudarBase(Mesh *scan, gsl_matrix *eixos)
{
	gsl_matrix *vec = gsl_matrix_calloc(3,1);
	gsl_matrix *transfvec = gsl_matrix_calloc(3,1);
	for(VertexIter it=scan->verts_begin();it!=scan->verts_end();it++)
	{
		gsl_matrix_set(vec,0,0,(*it)->a.g(0));
		gsl_matrix_set(vec,1,0,(*it)->a.g(1));
		gsl_matrix_set(vec,2,0,(*it)->a.g(2));
		gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.,eixos,vec,0.,transfvec);
		(*it)->a.g(0)=gsl_matrix_get(transfvec,0,0);
		(*it)->a.g(1)=gsl_matrix_get(transfvec,1,0);
		(*it)->a.g(2)=gsl_matrix_get(transfvec,2,0);
	}
}

void mudarBase(vector<Vertex *> *pontos, gsl_matrix *eixos)
{
	gsl_matrix *vec = gsl_matrix_calloc(3,1);
	gsl_matrix *transfvec = gsl_matrix_calloc(3,1);
	for(int i=0;i<pontos->size();i++)
	{
		gsl_matrix_set(vec,0,0,pontos->at(i)->a.g(0));
		gsl_matrix_set(vec,1,0,pontos->at(i)->a.g(1));
		gsl_matrix_set(vec,2,0,pontos->at(i)->a.g(2));
		gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.,eixos,vec,0.,transfvec);
		pontos->at(i)->a.g(0)=gsl_matrix_get(transfvec,0,0);
		pontos->at(i)->a.g(1)=gsl_matrix_get(transfvec,1,0);
		pontos->at(i)->a.g(2)=gsl_matrix_get(transfvec,2,0);
	}
}

gsl_matrix *inverseMatrix(gsl_matrix *m)
{
	int s;
	gsl_matrix *work = gsl_matrix_alloc(3,3);
	gsl_matrix *inv = gsl_matrix_alloc(3,3);
	gsl_permutation *p = gsl_permutation_alloc(3);
	gsl_matrix_memcpy( work, m );

    gsl_linalg_LU_decomp( work, p, &s );
	gsl_linalg_LU_invert( work, p, inv );

	gsl_permutation_free( p );
	gsl_matrix_free(work);
	return inv;
}

double estratificarScan(Mesh *scan, ScanStratification *estratificacao, int divisoes)
{
	gsl_matrix *matrizCovariancia=calcularMatrizCovariancia(scan);
	gsl_matrix *eixos=calcularEixosPrincipais(matrizCovariancia);
	printf("\nEixos Principais:");
	printf("\n%f %f %f",gsl_matrix_get(eixos,0,0),gsl_matrix_get(eixos,0,1),gsl_matrix_get(eixos,0,2));
	printf("\n%f %f %f",gsl_matrix_get(eixos,1,0),gsl_matrix_get(eixos,1,1),gsl_matrix_get(eixos,1,2));
	printf("\n%f %f %f",gsl_matrix_get(eixos,2,0),gsl_matrix_get(eixos,2,1),gsl_matrix_get(eixos,2,2));
	gsl_matrix *inversa=inverseMatrix(eixos);
	mudarBase(scan,inversa);
	janelagl->eixoX(0)=gsl_matrix_get(eixos,0,0);
	janelagl->eixoX(1)=gsl_matrix_get(eixos,1,0);
	janelagl->eixoX(2)=gsl_matrix_get(eixos,2,0);
	janelagl->eixoY(0)=gsl_matrix_get(eixos,0,1);
	janelagl->eixoY(1)=gsl_matrix_get(eixos,1,1);
	janelagl->eixoY(2)=gsl_matrix_get(eixos,2,1);
	janelagl->eixoZ(0)=gsl_matrix_get(eixos,0,2);
	janelagl->eixoZ(1)=gsl_matrix_get(eixos,1,2);
	janelagl->eixoZ(2)=gsl_matrix_get(eixos,2,2);
	
	double menorx=(*scan->verts_begin())->a.g(0);
	double maiorx=(*scan->verts_begin())->a.g(0);
	double menory=(*scan->verts_begin())->a.g(1);
	double maiory=(*scan->verts_begin())->a.g(1);

	for(VertexIter it=scan->verts_begin();it!=scan->verts_end();it++)
	{
		if((*it)->a.g(0)>maiorx)
			maiorx=(*it)->a.g(0);
		if((*it)->a.g(0)<menorx)
			menorx=(*it)->a.g(0);
		if((*it)->a.g(1)>maiory)
			maiory=(*it)->a.g(1);
		if((*it)->a.g(1)<menory)
			menory=(*it)->a.g(1);
	}

	double intervalox=(maiorx-menorx)/(float)divisoes;
	double intervaloy=(maiory-menory)/(float)divisoes;

	const gsl_rng_type *T;
	gsl_rng_env_setup();
	T = gsl_rng_default;
	gsl_rng * r;
	r = gsl_rng_alloc (T);
	unsigned long int max=gsl_rng_max(r);		
	unsigned long int min=gsl_rng_min(r);

	unsigned long int randnumber;
	double rand;
	double x,y,z,nx,ny,nz;

	for(double currx=menorx;currx<maiorx;currx+=intervalox)
		for(double curry=menory;curry<maiory;curry+=intervaloy)
		{
			randnumber=gsl_rng_get(r);
			rand=(double)(randnumber-min)/(double)(max-min);
			x=currx+0.5*intervalox+(rand-0.5)*intervalox*0.45;

			randnumber=gsl_rng_get(r);
			rand=(double)(randnumber-min)/(double)(max-min);
			y=curry+0.5*intervaloy+(rand-0.5)*intervaloy*0.45;
			bool achou=false;
			for(FaceIter it=scan->faces_begin();it!=scan->faces_end();it++)
				if(isInTriangle(x,y,*it,&z,&nx,&ny,&nz))
				{
					achou=true;
					break;
				}
			if(achou)
			{
				Vertex *v=new Vertex();
				v->a.g(0)=x;
				v->a.g(1)=y;
				v->a.g(2)=z;
				v->n.g(0)=nx;
				v->n.g(1)=ny;
				v->n.g(2)=nz;
				estratificacao->push_back(v);
			}
		}

	mudarBase(estratificacao,eixos);
	mudarBase(scan,eixos);
	
	gsl_rng_free(r);
	double temp=intervalox;
	if(intervaloy>temp)
		temp=intervaloy;
	return 2.*(0.5*temp*0.6)/scan->resolucao;
}

bool isInTriangle(double x, double y, Face *f, double *depth, double *nx, double *ny, double *nz)
{
	double A=f->hedge(0)->dst()->a.g(0)-f->hedge(2)->dst()->a.g(0);
	double B=f->hedge(1)->dst()->a.g(0)-f->hedge(2)->dst()->a.g(0);
	double C=f->hedge(2)->dst()->a.g(0)-x;
	double D=f->hedge(0)->dst()->a.g(1)-f->hedge(2)->dst()->a.g(1);
	double E=f->hedge(1)->dst()->a.g(1)-f->hedge(2)->dst()->a.g(1);
	double F=f->hedge(2)->dst()->a.g(1)-y;
	double G,H,I;
	G=H=I=0.;
	if(fabs(A)<0.000001 && fabs(B)<0.000001)
	{
		double temp=A;
		A=D;
		D=temp;
		temp=B;
		B=E;
		E=temp;
		temp=C;
		C=F;
		F=temp;
	}
	double u=(B*(F+I)-C*(E+H))/(A*(E+H)-B*(D+G));
	double v=(A*(F+I)-C*(D+G))/(B*(D+G)-A*(E+H));
	double w=1.-u-v;
	if(u<0. || u>1. || v<0. || v>1. || w<0. || w>1.)
		return false;
    
	*depth=u*f->hedge(0)->dst()->a.g(2)+v*f->hedge(1)->dst()->a.g(2)+w*f->hedge(2)->dst()->a.g(2);
	*nx=u*f->hedge(0)->dst()->n.g(0)+v*f->hedge(1)->dst()->n.g(0)+w*f->hedge(2)->dst()->n.g(0);
	*ny=u*f->hedge(0)->dst()->n.g(1)+v*f->hedge(1)->dst()->n.g(1)+w*f->hedge(2)->dst()->n.g(1);
	*nz=u*f->hedge(0)->dst()->n.g(2)+v*f->hedge(1)->dst()->n.g(2)+w*f->hedge(2)->dst()->n.g(2);
	return true;
}

double areaTriangulo(double x1, double y1, double x2, double y2, double x3, double y3)
{
	return (x2-x1)*(y3-y1)-(x3-x1)*(y2-y1);
}

char *extrairPath(char *fname)
{
	int pos=0;
	char *arq=fname;
	int i;
	for(int i=0;arq[i]!='\0';i++)
		if(arq[i]=='/')
			pos=i;
	char *arq2=new char[pos+2];
	int j;
	for(j=0;j<=pos;j++)
		arq2[j]=fname[j];
	arq2[j]='\0';
	return arq2;
}

double calcularResolucao(Mesh *m)
{
	DBLPRQUE arestas;
	for(EdgeIter it=m->edges_begin();it!=m->edges_end();it++)
	{
		double length=norm((*it)->org()->a.g-(*it)->dst()->a.g);
		arestas.push(length);
	}
	int mediana=arestas.size()/2;
	for(int i=0;i<mediana;i++)
		arestas.pop();
	double top=arestas.top();
	while(!arestas.empty())
		arestas.pop();
	m->resolucao=top;
	return top;
}

Mesh *abrirarq(char *newfile)
{
	if(newfile!=NULL)
	{
		Mesh *malha=new Mesh();
 		malha->read_ply(newfile);
		calcularResolucao(malha);
		return malha;
	}
	return NULL;
}

Vec3 transf_point(Vec3 p, gsl_matrix *m)
{
	Vec3 ponto;
	ponto(0)=p(0)*gsl_matrix_get(m,0,0)+p(1)*gsl_matrix_get(m,0,1)+p(2)*gsl_matrix_get(m,0,2)+gsl_matrix_get(m,0,3);
	ponto(1)=p(0)*gsl_matrix_get(m,1,0)+p(1)*gsl_matrix_get(m,1,1)+p(2)*gsl_matrix_get(m,1,2)+gsl_matrix_get(m,1,3);
	ponto(2)=p(0)*gsl_matrix_get(m,2,0)+p(1)*gsl_matrix_get(m,2,1)+p(2)*gsl_matrix_get(m,2,2)+gsl_matrix_get(m,2,3);
	return ponto;
}

Vec3 transf_normal(Vec3 n, gsl_matrix *m)
{
	Vec3 normal;
	normal(0)=n(0)*gsl_matrix_get(m,0,0)+n(1)*gsl_matrix_get(m,0,1)+n(2)*gsl_matrix_get(m,0,2);
	normal(1)=n(0)*gsl_matrix_get(m,1,0)+n(1)*gsl_matrix_get(m,1,1)+n(2)*gsl_matrix_get(m,1,2);
	normal(2)=n(0)*gsl_matrix_get(m,2,0)+n(1)*gsl_matrix_get(m,2,1)+n(2)*gsl_matrix_get(m,2,2);
	return normal;
}

ScanStratification *aplicarTransformacao(ScanStratification *scan, ScanStratification *previous_iter_scan, gsl_matrix *matriz4d)
{
	gsl_vector *p=gsl_vector_alloc(4);
	gsl_vector *qt=gsl_vector_alloc(4);
	ScanStratification *transf_strat=new ScanStratification();
	for(int i=0;i<scan->size();i++)
	{
		gsl_vector_set(qt,0,0.0);
		gsl_vector_set(qt,1,0.0);
		gsl_vector_set(qt,2,0.0);
		gsl_vector_set(qt,3,0.0);
		gsl_vector_set(p,0,scan->at(i)->a.g(0));
		gsl_vector_set(p,1,scan->at(i)->a.g(1));
		gsl_vector_set(p,2,scan->at(i)->a.g(2));
		gsl_vector_set(p,3,1.0);
		gsl_blas_dgemv(CblasNoTrans,1.,matriz4d,p,1.,qt);
		Vertex *newv=new Vertex();
		newv->nearest=scan->at(i)->nearest;
		if(previous_iter_scan!=NULL)
		{
			newv->previous=previous_iter_scan->at(i);
			previous_iter_scan->at(i)->next=newv;
		}
		newv->a.g(0)=gsl_vector_get(qt,0)/gsl_vector_get(qt,3);
		newv->a.g(1)=gsl_vector_get(qt,1)/gsl_vector_get(qt,3);
		newv->a.g(2)=gsl_vector_get(qt,2)/gsl_vector_get(qt,3);
		transf_strat->push_back(newv);
	}
	double t1,t2,t3;
	t1=gsl_matrix_get(matriz4d,0,3);
	t2=gsl_matrix_get(matriz4d,1,3);
	t3=gsl_matrix_get(matriz4d,2,3);

	gsl_matrix_set(matriz4d,0,3,0.);
	gsl_matrix_set(matriz4d,1,3,0.);
	gsl_matrix_set(matriz4d,2,3,0.);

	for(int i=0;i<scan->size();i++)
	{
		gsl_vector_set(qt,0,0.0);
		gsl_vector_set(qt,1,0.0);
		gsl_vector_set(qt,2,0.0);
		gsl_vector_set(qt,3,0.0);
		gsl_vector_set(p,0,scan->at(i)->n.g(0));
		gsl_vector_set(p,1,scan->at(i)->n.g(1));
		gsl_vector_set(p,2,scan->at(i)->n.g(2));
		gsl_vector_set(p,3,1.0);
		gsl_blas_dgemv(CblasNoTrans,1.,matriz4d,p,1.,qt);
		transf_strat->at(i)->n.g(0)=gsl_vector_get(qt,0)/gsl_vector_get(qt,3);
		transf_strat->at(i)->n.g(1)=gsl_vector_get(qt,1)/gsl_vector_get(qt,3);
		transf_strat->at(i)->n.g(2)=gsl_vector_get(qt,2)/gsl_vector_get(qt,3);
	}

	gsl_matrix_set(matriz4d,0,3,t1);
	gsl_matrix_set(matriz4d,1,3,t2);
	gsl_matrix_set(matriz4d,2,3,t3);
	delete qt;
	delete p;
	return transf_strat;
}

void abriraln(char *alnfile, Iteration *iteracao, bool first)
{
	printf("\Opening alignment %s...",alnfile);
	if(alnfile!=NULL)
	{
		FILE *alnf;
		if((alnf=fopen(alnfile, "r"))==NULL)
			return;

		char *path=extrairPath(alnfile);
		int qm;
		char temp[100];
		char nome[100];
		char pathaux[300];
		double d1,d2,d3,d4;
		fscanf(alnf,"%d",&qm);
		for(int i=0;i<qm;i++)
		{
			fscanf(alnf,"%s",nome);
			if(first)
			{
				printf("\nOpening scan %s...",nome);
				strcpy(pathaux,path);
				Mesh *mesh=abrirarq(strcat(pathaux,nome));
				scans.push_back(mesh);
			}
			gsl_matrix *m4d = gsl_matrix_calloc (4,4);
			fscanf(alnf,"%s",temp);
			fscanf(alnf,"%lf %lf %lf %lf",&d1,&d2,&d3,&d4);
			gsl_matrix_set(m4d,0,0,d1);
			gsl_matrix_set(m4d,0,1,d2);
			gsl_matrix_set(m4d,0,2,d3);
			gsl_matrix_set(m4d,0,3,d4);
			fscanf(alnf,"%lf %lf %lf %lf",&d1,&d2,&d3,&d4);
			gsl_matrix_set(m4d,1,0,d1);
			gsl_matrix_set(m4d,1,1,d2);
			gsl_matrix_set(m4d,1,2,d3);
			gsl_matrix_set(m4d,1,3,d4);
			fscanf(alnf,"%lf %lf %lf %lf",&d1,&d2,&d3,&d4);
			gsl_matrix_set(m4d,2,0,d1);
			gsl_matrix_set(m4d,2,1,d2);
			gsl_matrix_set(m4d,2,2,d3);
			gsl_matrix_set(m4d,2,3,d4);
			fscanf(alnf,"%lf %lf %lf %lf",&d1,&d2,&d3,&d4);
			gsl_matrix_set(m4d,3,0,d1);
			gsl_matrix_set(m4d,3,1,d2);
			gsl_matrix_set(m4d,3,2,d3);
			gsl_matrix_set(m4d,3,3,d4);
			iteracao->transformations->push_back(m4d);
		}
		printf("\nAlignment %s opened.",alnfile);
	}
}

void calcularVizinhos()
{
	for(int t=0;t<iteracoes.size();t++)
	{
		for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			(*it)->isNearest=false;

		printf("\nCalculando pontos mais proximos entre scans estratificados e reconstrucao da iteracao %d...",t);
		
		vector<Vertex *> *pontoskdtree = new vector<Vertex *>();
		for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			pontoskdtree->push_back(*it);
		Kdtree *kdtree = new Kdtree(pontoskdtree);
		double threshold=2*scans.at(0)->resolucao;
		for(int i=0;i<iteracoes.at(t)->stratifications->size();i++)
			for(int k=0;k<iteracoes.at(t)->stratifications->at(i)->size();k++)
			{
				Vertex *v=kdtree->searchkdtree(iteracoes.at(t)->stratifications->at(i)->at(k),threshold);
				if(v!=NULL)
				{
					iteracoes.at(t)->stratifications->at(i)->at(k)->nearest=v;
					v->isNearest=true;
				}
			}

		printf("\nCalculando pontos mais proximos entre reconstrucao e scans estratificados da iteracao %d...",t);
		
		vector<Vertex *> *pontoskdtreescan = new vector<Vertex *>();
		for(int s=0;s<iteracoes.at(t)->stratifications->size();s++)
			for(int k=0;k<iteracoes.at(t)->stratifications->at(s)->size();k++)
				pontoskdtreescan->push_back(iteracoes.at(t)->stratifications->at(s)->at(k));
		Kdtree *kdtreescans=new Kdtree(pontoskdtreescan);

		for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
		{
			Vertex *v=kdtreescans->searchkdtree(*it,threshold);
			if(v!=NULL)
				(*it)->nearest=v;
		}

		criarDrawableReconstruction(iteracoes.at(t),kdtree);
	}
	printf("\nOk.");
}

void criarDrawableReconstruction(Iteration *iteration, Kdtree *kdt)
{
	iteration->rec_strat=new RecStratification();
	vector<Vertex *> *not_drawable_rec=new vector<Vertex *>();
	set<Vertex *> *usedv=new set<Vertex *>();
	for(VertexIter it=iteration->reconstruction->verts_begin();it!=iteration->reconstruction->verts_end();it++)
	{
		if((*it)->isNearest && usedv->find(*it)==usedv->end())
		{
			vector<Vertex *> *U=new vector<Vertex *>();
			kdt->raiz->searchkdtree(*it,U,scans.at(0)->resolucao*reconstruction_threshold);
 			for(int i=0;i<U->size();i++)
				if(!U->at(i)->isNearest || usedv->find(U->at(i))!=usedv->end())
				{
					U->erase(U->begin()+i);
					i--;
				}
			if(U->size()>0)
			{
				for(int i=0;i<U->size();i++)
					usedv->insert(U->at(i));
				UnifiedVertex *rv=new UnifiedVertex();
				rv->v_union=U;
				Vertex *newv=new Vertex();
				for(int i=0;i<U->size();i++)
				{
					newv->a.g=newv->a.g+U->at(i)->a.g;
					newv->n.g=newv->n.g+U->at(i)->n.g;
				}
				newv->a.g=newv->a.g*(1./(float)U->size());
				newv->n.g=newv->n.g*(1./(float)U->size());
				rv->v=newv;
				iteration->rec_strat->push_back(rv);
			}
			else
				delete U;
		}
	}
}

void atualizarErros()
{
	for(int t=0;t<iteracoes.size();t++)
	{
		for(int i=0;i<iteracoes.at(t)->rec_strat->size();i++)
		{
			if(metodo==0)
			{
				float maior=-1.;
				for(int k=0;k<iteracoes.at(t)->rec_strat->at(i)->v_union->size();k++)
				{
					if(iteracoes.at(t)->rec_strat->at(i)->v_union->at(k)->erro>maior)
						maior=iteracoes.at(t)->rec_strat->at(i)->v_union->at(k)->erro;
				}
				iteracoes.at(t)->rec_strat->at(i)->v->erro=maior;
			}
			if(metodo==1)
			{
				float menor=99999999.;
				for(int k=0;k<iteracoes.at(t)->rec_strat->at(i)->v_union->size();k++)
				{
					if(iteracoes.at(t)->rec_strat->at(i)->v_union->at(k)->erro<menor)
						menor=iteracoes.at(t)->rec_strat->at(i)->v_union->at(k)->erro;
				}
				iteracoes.at(t)->rec_strat->at(i)->v->erro=menor;
			}
			if(metodo==2)
			{
				float soma=0.;
				for(int k=0;k<iteracoes.at(t)->rec_strat->at(i)->v_union->size();k++)
					soma+=iteracoes.at(t)->rec_strat->at(i)->v_union->at(k)->erro;
				iteracoes.at(t)->rec_strat->at(i)->v->erro=soma/(float)iteracoes.at(t)->rec_strat->at(i)->v_union->size();
			}
		}
	}
}

//Erro |Pi - Pi+1|
void callback_erro1(Fl_Widget *)
{
	if(!stratified)
	{
		printf("\nE necessario estratificar os scans.");
		return;
	}
	for(int t=0;t<iteracoes.size();t++)
        for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			(*it)->erro=-1;
	printf("\n\nErro Pi - Pi+1");
	for(int t=1;t<iteracoes.size();t++)
	{
		double somaerro=0.;
		int count=0;
        for(int m=0;m<iteracoes.at(t)->stratifications->size();m++)
			for(int k=0;k<iteracoes.at(t)->stratifications->at(m)->size();k++)
			{
				if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest!=NULL)
				{

					Vec3 dif=iteracoes.at(t)->stratifications->at(m)->at(k)->previous->a.g-iteracoes.at(t)->stratifications->at(m)->at(k)->a.g;
					float erro=norm(dif);
					if(erro>iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro)
					{
						if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro<0)
							count++;
						else
							somaerro-=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro;
						iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro=erro;
						somaerro+=erro;
					}

				}
			}
		printf("\nerro medio da iteracao %d: %lf",t,somaerro/(double)count);
	}
	atualizarErros();
	janelagl->iteracao=1;
	sliderIteracao->minimum(1);
	sliderIteracao->maximum(iteracoes.size()-1);
	sliderIteracao->value(1);
	sliderIteracao->redraw();
	janelagl->redraw();
	erroatual=1;
}


void callback_erro2(Fl_Widget *)
{
	if(!stratified)
	{
		printf("\E necessario estratificar os scans.");
		return;
	}
	for(int t=0;t<iteracoes.size();t++)
        for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			(*it)->erro=-1;
	printf("\n\nErro Si - Si+1");
	for(int t=1;t<iteracoes.size();t++)
	{
		double somaerro=0.;
		int count=0;
        for(int m=0;m<iteracoes.at(t)->stratifications->size();m++)
			for(int k=0;k<iteracoes.at(t)->stratifications->at(m)->size();k++)
			{
				if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest!=NULL && iteracoes.at(t)->stratifications->at(m)->at(k)->previous->nearest!=NULL)
				{
					Vec3 dif=iteracoes.at(t)->stratifications->at(m)->at(k)->previous->nearest->a.g-iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->a.g;
					float erro=norm(dif);
					if(erro>iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro)
					{
						if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro<0)
							count++;
						else
							somaerro-=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro;
						iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro=erro;
						somaerro+=erro;
					}
				}
			}
		printf("\nerro medio da iteracao %d: %lf",t,somaerro/(double)count);
	}
	atualizarErros();
	janelagl->iteracao=1;
	sliderIteracao->minimum(1);
	sliderIteracao->maximum(iteracoes.size()-1);
	sliderIteracao->value(1);
	sliderIteracao->redraw();
	janelagl->redraw();
	erroatual=2;
}

void callback_erro3(Fl_Widget *)
{
	if(!stratified)
	{
		printf("\E necessario estratificar os scans.");
		return;
	}
	for(int t=0;t<iteracoes.size();t++)
        for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			(*it)->erro=-1;
	printf("\n\nErro (Pi-Si)-(Pi+1-Si+1)");
	for(int t=1;t<iteracoes.size();t++)
	{
		double somaerro=0.;
		int count=0;
		for(int m=0;m<iteracoes.at(t)->stratifications->size();m++)
			for(int k=0;k<iteracoes.at(t)->stratifications->at(m)->size();k++)
			{
				if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest!=NULL && iteracoes.at(t)->stratifications->at(m)->at(k)->previous->nearest!=NULL && iteracoes.at(t)->stratifications->at(m)->at(k)->nearest!=NULL)
				{
					Vec3 dif1=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->a.g-iteracoes.at(t)->stratifications->at(m)->at(k)->a.g;
					Vec3 dif2=iteracoes.at(t)->stratifications->at(m)->at(k)->previous->nearest->a.g-iteracoes.at(t)->stratifications->at(m)->at(k)->previous->a.g;
					float erro=fabs(norm(dif1)-norm(dif2));
					if(erro>iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro)
					{
						if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro<0)
							count++;
						else
							somaerro-=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro;
						iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro=erro;
						somaerro+=erro;
					}
				}
			}
		printf("\nerro medio da iteracao %d: %lf",t,somaerro/(double)count);
	}
	atualizarErros();
	janelagl->iteracao=1;
	sliderIteracao->minimum(1);
	sliderIteracao->maximum(iteracoes.size()-1);
	sliderIteracao->value(1);
	sliderIteracao->redraw();
	
	janelagl->redraw();
	erroatual=3;
}

void callback_erro4(Fl_Widget *)
{
	if(!stratified)
	{
		printf("\E necessario estratificar os scans.");
		return;
	}
	for(int t=0;t<iteracoes.size();t++)
        for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			(*it)->erro=-1.;
	printf("\n\nErro Si-Pi");

	for(int t=0;t<iteracoes.size();t++)
	{
		double somaerro=0.;
		int count=0;
        for(int m=0;m<iteracoes.at(t)->stratifications->size();m++)
			for(int k=0;k<iteracoes.at(t)->stratifications->at(m)->size();k++)
			{
				if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest!=NULL)
				{
					Vec3 dif=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->a.g-iteracoes.at(t)->stratifications->at(m)->at(k)->a.g;
					float erro=norm(dif);
					if(erro>iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro)
					{
						if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro<0)
							count++;
						else
							somaerro-=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro;
						iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro=erro;
						somaerro+=erro;
					}
				}
			}
		printf("\nerro medio da iteracao %d: %lf",t,somaerro/(double)count);
	}
	atualizarErros();
	janelagl->iteracao=0;
	sliderIteracao->minimum(0);
	sliderIteracao->maximum(iteracoes.size()-1);
	sliderIteracao->value(0);
	sliderIteracao->redraw();
	janelagl->redraw();
	erroatual=4;
}

void callback_erro5(Fl_Widget *)
{
	if(!stratified)
	{
		printf("\E necessario estratificar os scans.");
		return;
	}
	for(int t=0;t<iteracoes.size();t++)
        for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			(*it)->erro=-1;
	printf("\n\nErro Pi - Qi");

	for(int t=0;t<iteracoes.size();t++)
	{
		double somaerro=0.;
		int count=0;
        for(int m=0;m<iteracoes.at(t)->stratifications->size();m++)
			for(int k=0;k<iteracoes.at(t)->stratifications->at(m)->size();k++)
			{
				if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest!=NULL && iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->nearest!=NULL)
				{
					Vec3 dif=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->nearest->a.g-iteracoes.at(t)->stratifications->at(m)->at(k)->a.g;
					float erro=norm(dif);
					if(erro>iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro)
					{
						if(iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro<0)
							count++;
						else
							somaerro-=iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro;
						iteracoes.at(t)->stratifications->at(m)->at(k)->nearest->erro=erro;
						somaerro+=erro;
					}
				}
			}
		printf("\nerro medio da iteracao %d: %lf",t,somaerro/(double)count);
	}
	atualizarErros();
	janelagl->iteracao=0;
	sliderIteracao->minimum(0);
	sliderIteracao->maximum(iteracoes.size()-1);
	sliderIteracao->value(0);
	sliderIteracao->redraw();
	
	janelagl->redraw();
	erroatual=5;
}


void callback_openiteration(Fl_Widget *)
{
	if(stratified)
	{
		printf("\nCannot open iteration after stratification.");
		return;
	}
	char *alnfile = fl_file_chooser("Escolha o arquivo com alinhamento dos scans", "*.aln", NULL);
	vector<Mesh *> *malhas=NULL;
	Mesh *rec=NULL;
	Iteration *new_iter;
	if(alnfile!=NULL)
	{
		new_iter=new Iteration();
		bool first=false;
		if(iteracoes.size()==0)
			first=true;
		abriraln(alnfile,new_iter,first);

	}
	else
		return;
	char *recfile = fl_file_chooser("Escolha o arquivo com a reconstrucao correspondente", "*.ply", NULL);
	if(recfile!=NULL)
		rec=abrirarq(recfile);
	else
		return;
	new_iter->reconstruction=rec;
	iteracoes.push_back(new_iter);	
	printf("\nOk.");
}

void slider_callback(Fl_Widget *)
{
	janelagl->iteracao=((Fl_Value_Slider *)sliderIteracao)->value();
	
	janelagl->redraw();
}


void sliderthreshold_callback(Fl_Widget *)
{
	reconstruction_threshold=((Fl_Value_Slider *)sliderThreshold)->value();

	for(int t=0;t<iteracoes.size();t++)
	{
		vector<Vertex *> *pontosreccomerro = new vector<Vertex *>();
		for(VertexIter it=iteracoes.at(t)->reconstruction->verts_begin();it!=iteracoes.at(t)->reconstruction->verts_end();it++)
			pontosreccomerro->push_back(*it);
		Kdtree *kdtree3=new Kdtree(pontosreccomerro);
		criarDrawableReconstruction(iteracoes.at(t),kdtree3);
	}
	if(erroatual==1)
		callback_erro1(0);
	if(erroatual==2)
		callback_erro2(0);
	if(erroatual==3)
		callback_erro3(0);
	if(erroatual==4)
		callback_erro4(0);
	if(erroatual==5)
		callback_erro5(0);
	janelagl->redraw();
}

void slidererror_callback(Fl_Widget *)
{
	janelagl->maxerror=((Fl_Value_Slider *)sliderError)->value();
	
	janelagl->redraw();
}

void sliderdisk_callback(Fl_Widget *)
{
	janelagl->disksize=((Fl_Value_Slider *)sliderDisk)->value();
	
	janelagl->redraw();
}

void callback_malhas(Fl_Widget *)
{
	janelagl->desenharMalhas=cbMalhas->value();
	janelagl->redraw();
}

void callback_rec(Fl_Widget *)
{
	janelagl->desenharReconstrucao=cbRec->value();
	janelagl->redraw();
}

void rbcall(Fl_Widget *fw)
{
	rb1->value(false);
	rb2->value(false);
	((Fl_Round_Button *)fw)->value(true);
	if(rb1->value())
		janelagl->drawdisks=true;
	if(rb2->value())
		janelagl->drawdisks=false;
	
	janelagl->redraw();
}

void callback_estratificar(Fl_Widget *)
{
	if(iteracoes.size()==0)
	{
		printf("\nAbra uma iteracao antes.");
		return;
	}
	estrat_param*=0.7;
	double thresh=0.;
	double temp;
	double threshold=3*scans.at(0)->resolucao;
	for(int i=0;i<iteracoes.size();i++)
	{
		for(int j=0;j<iteracoes.at(i)->stratifications->size();j++)
			delete iteracoes.at(i)->stratifications->at(j);
		iteracoes.at(i)->stratifications->clear();
	}

	for(int m=0;m<scans.size();m++)
	{
		printf("\nEstratificando scan %s...",scans.at(m)->arquivo);
		ScanStratification *estratificacao=new ScanStratification();
		if((temp=estratificarScan(scans.at(m),estratificacao,sqrt(estrat_param*scans.at(m)->num_verts())))>thresh)
			thresh=temp;
		iteracoes.at(0)->stratifications->push_back(aplicarTransformacao(estratificacao,NULL,iteracoes.at(0)->transformations->at(m)));
		for(int i=1;i<iteracoes.size();i++)
			iteracoes.at(i)->stratifications->push_back(aplicarTransformacao(estratificacao,iteracoes.at(i-1)->stratifications->at(m),iteracoes.at(i)->transformations->at(m)));
	}
	stratified=true;
	calcularVizinhos();
	((Fl_Value_Slider *)sliderThreshold)->value(thresh);
	sliderthreshold_callback(0);
	((Fl_Value_Slider *)sliderThreshold)->redraw();
	((Fl_Value_Slider *)sliderDisk)->value(thresh*scans.at(0)->resolucao);
	sliderdisk_callback(0);
	((Fl_Value_Slider *)sliderDisk)->redraw();
	janelagl->iteracoes=&iteracoes;
	if(erroatual==1)
		callback_erro1(0);
	if(erroatual==2)
		callback_erro2(0);
	if(erroatual==3)
		callback_erro3(0);
	if(erroatual==4)
		callback_erro4(0);
	if(erroatual==5)
		callback_erro5(0);

	janelagl->redraw();

}

void createwindow()
{
	window = new Fl_Double_Window(50, 20, 1000, 680, "Error Visualization - Thales Vieira");

	janelagl=new mywindow(20, 10, 700, 620, "OpenGL");
	btnOpenIter=new Fl_Button(770, 20, 220, 25, "Open Iteration...");
	btnErro1=new Fl_Button(770, 80, 220, 25, "View Error Pi-Pi+1");
	btnErro2=new Fl_Button(770, 110, 220, 25, "View Error Si-Si+1");
	btnErro3=new Fl_Button(770, 140, 220, 25, "View Error |Pi-Si|-|Pi+1-Si+1|");
	btnErro4=new Fl_Button(770, 170, 220, 25, "View Error Pi-Si");
	btnErro5=new Fl_Button(770, 200, 220, 25, "View Error Pi-Qi");
	btnEstratificar=new Fl_Button(770, 50, 220, 25, "Stratify");

	sliderIteracao=new Fl_Value_Slider(20,640,700,20,"Iteration");
	sliderIteracao->type(FL_HOR_NICE_SLIDER);
	sliderIteracao->minimum(0);
	sliderIteracao->maximum(0);
	sliderIteracao->callback(slider_callback);
	sliderIteracao->value(0);
	sliderIteracao->step(1);

	sliderError=new Fl_Value_Slider(730,330,260,20,"Max Error");
	sliderError->type(FL_HOR_NICE_SLIDER);
	sliderError->minimum(0.0001);
	sliderError->maximum(0.3);
	sliderError->callback(slidererror_callback);
	sliderError->value(0.04);
	sliderError->step(0.001);	
	
	sliderDisk=new Fl_Value_Slider(730,370,260,20,"Radius");
	sliderDisk->type(FL_HOR_NICE_SLIDER);
	sliderDisk->minimum(0.0001);
	sliderDisk->maximum(0.12);
	sliderDisk->callback(sliderdisk_callback);
	sliderDisk->value(0.028);
	sliderDisk->step(0.001);	

	sliderThreshold=new Fl_Value_Slider(730,410,260,20,"Reconstruction Resampling Threshold");
	sliderThreshold->type(FL_HOR_NICE_SLIDER);
	sliderThreshold->minimum(0.1);
	sliderThreshold->maximum(5.);
	sliderThreshold->callback(sliderthreshold_callback);
	sliderThreshold->value(0.5);
	sliderThreshold->step(0.05);

	
	boxgrupo=new Fl_Box(810, 460, 120, 75,"");
	boxgrupo->box(FL_EMBOSSED_BOX);
	boxel=new Fl_Box(856, 440, 30, 70,"Draw Points As:");
	boxel->box(FL_NO_BOX);
	rb1=new Fl_Round_Button(820, 480, 160, 30, "disks");
	rb2=new Fl_Round_Button(820, 500, 160, 30, "spheres");
	rb1->callback(rbcall);
	rb2->callback(rbcall);
	rb1->value(true);
	rb2->value(false);

	cbMalhas=new Fl_Check_Button(785, 265, 160, 25, "Show Scans");
	cbRec=new Fl_Check_Button(785, 290, 160, 25, "Show Reconstruction");
	cbRec->value(true);
	cbMalhas->value(false);
	btnOpenIter->callback(callback_openiteration);
	btnErro1->callback(callback_erro1);
	btnErro2->callback(callback_erro2);
	btnErro3->callback(callback_erro3);
	btnErro4->callback(callback_erro4);
	btnErro5->callback(callback_erro5);
	btnEstratificar->callback(callback_estratificar);
	cbMalhas->callback(callback_malhas);
	cbRec->callback(callback_rec);
	janelagl->redraw();
}

int main()
{
	stratified=false;
	estrat_param=1.;
	metodo=0;
	erroatual=-1;
	reconstruction_threshold=0.5;
	createwindow();	
	janelagl->iteracoes=&iteracoes;
	window->begin();
	window->end();
	window->show();
	return Fl::run();
}