#ifndef MESH2_H
#define MESH2_H

//typedef unsigned char uchar;

#include "CImg.h"

#include <vector>
#include "a48.h"

typedef unsigned char uchar;

using namespace std;
using namespace cimg_library;

namespace A48 {


class Mesh {
  FaceContainer   fc_;   // faces
  EdgeContainer   ec_;   // edges
  VertexContainer vc_;   // vertices


 public:
	 char *arquivo;
	 double resolucao;
	 bool isrec;

	 vector<Vertex *> *estratificacao;
   Vec3 scannerPos;
  Mesh();
  ~Mesh();

  VertexIter verts_begin(){ return vc_.begin(); };
  VertexIter verts_end(){ return vc_.end(); };
  int num_verts() { return vc_.size(); };

  EdgeIter edges_begin() { return ec_.begin(); };
  EdgeIter edges_end() { return ec_.end(); };
  int num_edges() { return ec_.size(); };

  FaceIter faces_begin() { return fc_.begin(); };
  FaceIter faces_end() { return fc_.end(); };
  int num_faces() { return fc_.size(); };
	bool add_vertex(Vertex *v) 
	{ std::pair<VertexIter, bool> r = vc_.insert(v); return r.second; };
  void read_ply(char *fname);
  void read_texture(char *rangename, CImg<uchar> *imagem);
	void write_off(char *file);
  void clear_vertex() { vc_.clear(); }
  double *bbox();
  Vec3 baricentro;
  void calcularCentro();
  bool add_edge(Edge *e) 
    { std::pair<EdgeIter, bool> r = ec_.insert(e);  return r.second;};
  bool add_face(Face* f) 
    { std::pair<FaceIter, bool> r = fc_.insert(f); return r.second;};
  Mesh *copiar();

 private:
 void setColorVert(Vec3 *vertice, double R, double G, double B);
  Vertex *add_vertex();
  
  void del_vertex(Vertex* v) { vc_.erase(v); delete(v); };

  Hedge *add_edge(Vertex *v0, Vertex *v1);
  
  void del_edge(Edge* e) { ec_.erase(e); delete(e); }

  Face* add_face(Hedge* e0, Hedge* e1, Hedge* e2);
  
  void del_face(Face* f) { fc_.erase(f); delete(f); }

  void link_mesh();

  //  void set_base_mesh(int np, int* tris, int nf);
  Hedge* get_hedge(int i0, int i1, Vertex* verts[], HedgeMap* edges);
  double calcularAMixed(Vertex *v);
  bool obtuseTriangle(Vertex *v1, Vertex *v2, Vertex *v3);
  bool obtuseAngle(Vertex *v1, Vertex *v2, Vertex *v3);
  double triangleArea(Vertex *v1, Vertex *v2, Vertex *v3); 
  double voronoiRegion(Vertex *v, Vertex *v1, Vertex *v2);
  bool inverterNormal(Vec3 K, Vertex *v);
 public:

	


  void put_vertex(Vertex *v) { add_vertex(v); };
  void put_face(int i0, int i1, int i2, Vertex* verts[], HedgeMap* edges);
  void create_normals();
  void create_normals_curv();
  void create_weights();
  bool contem_Aresta(vector<Hedge *> *vec, Hedge *h);
  char *extrairArquivo(char *fname);
};


}

#endif
