/*:
**     mesh.cc 
*/

//#include "a48.h"
#include "mesh.h"
#include <map>

using namespace A48;

Mesh::Mesh()
{
	estratificacao=NULL;
}


Mesh::~Mesh()
{
  for (VertexIter p = vc_.begin(); p != vc_.end(); p++) 
    delete *p;
  vc_.clear();
  for (EdgeIter e = ec_.begin(); e != ec_.end(); e++)
    delete *e;
  ec_.clear();
  for (FaceIter f = fc_.begin(); f != fc_.end(); f++)
    delete *f;
  fc_.clear();
  delete arquivo;
}

Mesh *Mesh::copiar()
{
	Mesh *m=new Mesh();
	for (VertexIter p = vc_.begin(); p != vc_.end(); p++)
	{
		Vertex *v=new Vertex();
		v->a.g=(*p)->a.g;
		m->put_vertex(v);
	}
	for (EdgeIter e = ec_.begin(); e != ec_.end(); e++)
	{
		m->add_edge(*e);
	}	
	for (FaceIter f = fc_.begin(); f != fc_.end(); f++)
	{
		m->add_face(*f);
	}	
	return m;
}


Face* Mesh::add_face(Hedge *e0, Hedge *e1, Hedge *e2)
{
  Face *f = new Face(e0, e1, e2);
  add_face(f);
  return f;
}

Hedge* Mesh::add_edge(Vertex *v0, Vertex *v1)
{
  Edge *e = new Edge(v0, v1);
  add_edge(e);
  return e->hedge(0);
}

Vertex* Mesh::add_vertex(void)
{
  Vertex *v = new Vertex();
  add_vertex(v);
  return v;
}

void Mesh::link_mesh()
{
  for (FaceIter f = fc_.begin(); f != fc_.end(); f++)
    (*f)->link_star_verts();
  for (EdgeIter e = ec_.begin(); e != ec_.end(); e++) {
    Hedge* h = (*e)->hedge(0);
    if (h->is_bdry())
      h->dst()->set_star(h);
  }
}

Hedge* Mesh::get_hedge(int i0, int i1, Vertex* verts[], HedgeMap *hedges)
{
  bool mate = false;
  if (i0 > i1) {
    std::swap<int>(i0, i1);
    mate = true;
  }
  Hedge* he; HedgeMap::iterator ei  = hedges->find(Ipair(i0,i1));
  if (ei == hedges->end())
    (*hedges)[Ipair(i0,i1)] = he = add_edge(verts[i0], verts[i1]);
  else
    he = (*ei).second;
  return (mate)? he->mate() : he ;
}

void Mesh::put_face(int i0, int i1, int i2, Vertex* verts[], HedgeMap* hedges)
{
  Hedge* e0 = get_hedge(i1, i2, verts, hedges);
  Hedge* e1 = get_hedge(i2, i0, verts, hedges);
  Hedge* e2 = get_hedge(i0, i1, verts, hedges);
  add_face(e0, e1, e2);
}

/*
void Mesh::set_base_mesh(int np, int* tris, int nf)
{
  Vertex** verts = new (Vertex*)[np];
  for (int i=0; i < np; i++) {
    verts[i] = new Vertex();
    add_vertex(verts[i]);
  }
  HedgeMap hedges;
  for (int i=0; i < nf; i++, tris += 3) 
    put_face(tris[2], tris[0],tris[1], verts, &hedges);
  link_mesh();
  delete(verts);
}
*/

char *Mesh::extrairArquivo(char *fname)
{
	int pos=0;
	char *arq=fname;
	int i;
	for(i=0;arq[i]!='\0';i++)
		if(arq[i]=='/')
			pos=i+1;
	char *arq2=new char[i-pos+1];
	int j;
	for(j=0;j<i-pos;j++)
		arq2[j]=fname[pos+j];
	arq2[j]='\0';
	return arq2;
}

void Mesh::read_texture(char *rangename, CImg<uchar> *imagem)
{
	
	FILE *arq;
	if( (arq  = fopen( (const char*) rangename, "r" )) == NULL )
		printf( "The range grid file was not opened\n" );
	else
	{
		char *actual_str=new char[100];
		long actual_long;
		long num_cols,num_rows,num_vert;
		fscanf(arq,"%s",actual_str);
		//LEITURA DO HEADER
		
		while(strcmp(actual_str,"end_header")!=0)
		{
			if(strcmp(actual_str,"num_cols")==0)
			{
				fscanf(arq,"%d",&actual_long);
				num_cols=actual_long;
			}
			if(strcmp(actual_str,"num_rows")==0)
			{
				fscanf(arq,"%d",&actual_long);
				num_rows=actual_long;
			}
			if(strcmp(actual_str,"vertex")==0)
			{
				fscanf(arq,"%d",&actual_long);
				num_vert=actual_long;
			}
			fscanf(arq,"%s",actual_str);
		}
		Vec3 *vertices=new Vec3[num_vert];
		for(int i=0;i<num_vert;i++)
		{
			float x,y,z;
			fscanf(arq,"%f %f %f",&x,&y,&z);
			vertices[i](0)=x;
			vertices[i](1)=y;
			vertices[i](2)=z;
		}	
		for(int i=0;i<num_rows;i++)
			for(int j=0;j<num_cols;j++)
			{
				fscanf(arq,"%d",&actual_long);
				if(actual_long==1)
				{
					fscanf(arq,"%d",&actual_long);
					setColorVert(&vertices[actual_long],(*imagem)(j,i,0,0),(*imagem)(j,i,0,1),(*imagem)(j,i,0,2));
				}
			}				
	}
}

void Mesh::setColorVert(Vec3 *vertice, double R, double G, double B)
{
	double menor=norm(*vertice-(*verts_begin())->a.g);
	Vertex *menorvert=*verts_begin();
	for(VertexIter it=verts_begin();it!=verts_end();it++)	
		if(norm(*vertice-(*it)->a.g)<menor)
		{
			menor=norm(*vertice-(*it)->a.g);
			menorvert=*it;
		}
	menorvert->cor.g(0)=R/256.;
	menorvert->cor.g(1)=G/256.;
	menorvert->cor.g(2)=B/256.;
}

void Mesh::read_ply(char * fname)
{
	arquivo=extrairArquivo(fname);
	ply_read_file(fname, this);

	link_mesh();
	create_normals();
	create_normals_curv();

	calcularCentro();
	create_weights();
}

void Mesh::calcularCentro()
{
	double *bbox=this->bbox();
	baricentro=Vec3((bbox[1]-bbox[0])*0.5,(bbox[3]-bbox[2])*0.5,(bbox[5]-bbox[4])*0.5);
	delete bbox;
}

void Mesh::write_off(char *file){

if (file != NULL){
		FILE *arquivo;
		if( (arquivo  = fopen(file, "w" )) == NULL )
      printf( "The file 'data' was not opened\n" );
		else{
			int pid = 0;
			
			fprintf(arquivo,"%d %d\n", num_verts(), num_faces());
			for (VertexIter v = vc_.begin(); v != vc_.end(); v++) {				
				fprintf(arquivo,"%g %g %g\n",  (*v)->a.g[X], (*v)->a.g[Y], (*v)->a.g[Z]);
				(*v)->id = pid++;
			}
			for (FaceIter f = fc_.begin(); f != fc_.end(); f++) {
					fprintf(arquivo,"%d %d %d\n", (*f)->vertex(0)->id, (*f)->vertex(1)->id,(*f)->vertex(2)->id); 
			}


			fclose(arquivo);
		}	
	
}


}


double *Mesh::bbox()
{
	double *box=new double[6];
	box[0]=box[1]=(*vc_.begin())->a.g(0);
	box[2]=box[3]=(*vc_.begin())->a.g(1);
	box[4]=box[5]=(*vc_.begin())->a.g(2);
	for (VertexIter v = vc_.begin(); v != vc_.end(); v++)
	{
		if((*v)->a.g(0)>box[1])
			box[1]=(*v)->a.g(0);
		if((*v)->a.g(0)<box[0])
			box[0]=(*v)->a.g(0);
		if((*v)->a.g(1)>box[3])
			box[3]=(*v)->a.g(1);
		if((*v)->a.g(1)<box[2])
			box[2]=(*v)->a.g(1);
		if((*v)->a.g(2)>box[5])
			box[5]=(*v)->a.g(2);
		if((*v)->a.g(2)<box[4])
			box[4]=(*v)->a.g(2);
	}
	return box;
}

bool Mesh::contem_Aresta(vector<Hedge *> *vec, Hedge *h)
{
	for(int i=0;i<vec->size();i++)
		if(vec->at(i)==h)
			return true;
	return false;
}

void Mesh::create_normals()
{
	for(FaceIter fit=faces_begin();fit!=faces_end();fit++)
		(*fit)->criarNormal();
	for (VertexIter v = vc_.begin(); v != vc_.end(); v++)
	{
		Vertex *vert=(*v);
		Hedge *h=(*v)->star_first();
		Vec3 normal=Vec3(0,0,0);
		vector<Hedge *> *usados = new vector<Hedge *>();
		for(;h && h->face() && !contem_Aresta(usados,h);h=(*v)->star_next(h))
		{
			normal+=h->face()->getNormal();
			usados->push_back(h);
		}
		if(unitize(normal)==0)
		{
			normal=Vec3(0,0,0);
			if((*v)->star_first()!=NULL)
				if((*v)->star_first()->mate()->face()!=NULL)
					normal+=(*v)->star_first()->mate()->face()->getNormal();
				else
					normal+=(*v)->star_first()->mate()->org()->n.g;
		}
		if(unitize(normal)==0)
		{
			normal=Vec3(0,0,0);
			Hedge *h=(*v)->star_first();
			Vertex *vert=(*v);
			usados->clear();
			for(;h && h->face() && !contem_Aresta(usados,h);h=(*v)->star_next(h))
			{
				normal+=h->org()->n.g;
				usados->push_back(h);
			}
		}	
		unitize(normal);
		(*v)->n.g=normal;
		usados->clear();
		delete usados;
	}
}

void Mesh::create_normals_curv()               //calcula normal e curvatura media em cada ponto
{
	for (VertexIter v = vc_.begin(); v != vc_.end(); v++)
	{
		
		bool valid_vert=true;
		Hedge *h=(*v)->star_first();
		if(h==NULL)
			valid_vert=false;
		int counter=0;
		for(; h;h=(*v)->star_next(h))         // verifica validade da star
		{
			if(h->face()==NULL)
				valid_vert=false;
			counter++;
			if(counter>10)
			{
				valid_vert=false;
				break;
			}
		}
		if(valid_vert)
		{
			Vertex *v1,*v2;
			double AM=calcularAMixed(*v);
			h=(*v)->star_first();
			v2=h->org();
			Vertex *anterior;
			Vertex *atual;
			Vertex *proximo;
			for(Hedge *h2=h;h2;h2=(*v)->star_next(h2))
				atual=h2->org();
			Vec3 K=Vec3(0,0,0);
			double somatheta=0;
			for(; h;h=(*v)->star_next(h))
			{
				anterior=atual;
				atual=h->org();
				
				if((*v)->star_next(h)==NULL)
					proximo=(*v)->star_first()->org();
				else
					proximo=(*v)->star_next(h)->org();

				// CALCULO DE H
				Vec3 a = (*v)->a.g - anterior->a.g;
				Vec3 b = atual->a.g - anterior->a.g;
				double angalfa=acos((a*b)/(norm(a)*norm(b)));
				a = (*v)->a.g - proximo->a.g;
				b = atual->a.g - proximo->a.g;
				double angbeta=acos((a*b)/(norm(a)*norm(b)));
				K+=(1.0/tan(angalfa) + 1.0/tan(angbeta))*(atual->a.g-(*v)->a.g);
				
				// CALCULO DE G

				a = atual->a.g - (*v)->a.g;
				b = proximo->a.g - (*v)->a.g;
				somatheta+=acos((a*b)/(norm(a)*norm(b)));
			}
			K/=(2*AM);
			(*v)->H=0.5*norm(K);
			(*v)->G=(2*M_PI-somatheta)/AM;
			if(inverterNormal(K,(*v)))
			{
				(*v)->H*=-1;
				K*=-1;
			}
			double delta=pow((*v)->H,2)-(*v)->G;
			if(delta<0.0) delta=0.0;
			(*v)->k1=(*v)->H+sqrt(delta);
			(*v)->k2=(*v)->H-sqrt(delta);
			(*v)->k12k22=pow((*v)->k1,2)+pow((*v)->k2,2);
		//	printf("\ncurv: %f %f",(*v)->k1,(*v)->k2); 
			(*v)->hasH=(*v)->k12k22>=0;
			if(norm((*v)->n.g)<0.9)
				(*v)->n.g=K/norm(K);
		}
		else
		{
	/*		if((*v)->star_first())
			{
				if((*v)->star_first()->face())
					(*v)->n.g=(*v)->star_first()->face()->getNormal();
			}
			else
				(*v)->n.g= Vec3(0,0,0);*/
			(*v)->hasH=false;
		}
	}
}

double Mesh::calcularAMixed(Vertex *v)
{
	double AM=0;
	Vertex *v1,*v2;
	Hedge *h=v->star_first();
	v2=h->org();
	Vertex *primeiro=h->org();
	h=v->star_next(h);
	for(; h;h=v->star_next(h))  
	{
		v1=v2;
		v2=h->org();
		if(!obtuseTriangle(v,v1,v2))
		{
			AM+=voronoiRegion(v,v1,v2);
		}
		else
			if(obtuseAngle(v1,v,v2))
				AM+=0.5*triangleArea(v,v1,v2);
			else
				AM+=0.25*triangleArea(v,v1,v2);
	}
	v1=v2;
	v2=primeiro;
	if(!obtuseTriangle(v,v1,v2))
		{
			AM+=voronoiRegion(v,v1,v2);
		}
		else
			if(obtuseAngle(v1,v,v2))
				AM+=0.5*triangleArea(v,v1,v2);
			else
				AM+=0.25*triangleArea(v,v1,v2);

	return AM;
}

bool Mesh::obtuseTriangle(Vertex *v1, Vertex *v2, Vertex *v3)
{
	Vec3 a = v2->a.g - v1->a.g;
	Vec3 b = v3->a.g - v1->a.g;
	if(a*b<0)
		return true;

	a = v1->a.g - v2->a.g;
	b = v3->a.g - v2->a.g;
	if(a*b<0)
		return true;

	a = v2->a.g - v3->a.g;
	b = v1->a.g - v3->a.g;
	if(a*b<0)
		return true;

	return false;	
}

bool Mesh::obtuseAngle(Vertex *v1, Vertex *v2, Vertex *v3) //retorna verdadeiro se o angulo V1V2V3 for obtuso
{
	Vec3 a = v1->a.g - v2->a.g;
	Vec3 b = v3->a.g - v2->a.g;
	if(a*b<0)
		return true;

	return false;	
}

double Mesh::triangleArea(Vertex *v1, Vertex *v2, Vertex *v3)
{
	Vec3 a = v1->a.g - v2->a.g;
	Vec3 b = v3->a.g - v2->a.g;
	return 0.5*norm(a^b);
}

bool Mesh::inverterNormal(Vec3 K, Vertex *v)
{
	Vec3 n=v->star_first()->face()->getNormal();
	return n*K<0;
//	double d=-n*v->a.g;
//	Vec3 Kun=K/norm(K);
//	Vec3 pontoteste=v->a.g+100*Kun;
//	return pontoteste*n+d<0;
}

double Mesh::voronoiRegion(Vertex *v, Vertex *v1, Vertex *v2)
{
	Vec3 a = v->a.g - v1->a.g;
	Vec3 b = v2->a.g - v1->a.g;
	double angv1=acos((a*b)/(norm(a)*norm(b)));
	a = v->a.g - v2->a.g;
	b = v1->a.g - v2->a.g;
	double angv2=acos((a*b)/(norm(a)*norm(b)));
	return (1.0/tan(angv1)*pow(norm(v->a.g-v2->a.g),2) + 1.0/tan(angv2)*pow(norm(v->a.g-v1->a.g),2))/8.0;
}

void Mesh::create_weights()
{
	for (VertexIter v = vc_.begin(); v != vc_.end(); v++)
	{
		if(norm2((*v)->n.g)<0.9)
			(*v)->weight=0;
		else
		{
			Vec3 dir=scannerPos-(*v)->a.g;
			unitize(dir);
			(*v)->weight=dir*(*v)->n.g;
		}
	}
}