#include ".\mywindow.h"
#include "colormaps.h"
#define BUF_SIZE 1024
#define PICK_REGION_SIZE 20

mywindow::mywindow(int X, int Y, int W, int H, const char *L) : Fl_Gl_Window(X, Y, W, H, L)
{
	malhas=NULL;
	reconstructed=NULL;	
	desenharMalhas=false;
	desenharReconstrucao=true;
	scroll=0;
	ultimomouse=Vec3(0,0,0);
	desenharPontos=false;
	iluminacaoPorVertice=true;
	baricentro=Vec3(0,0,0);
	qmalhas=0;
	vel=0.1;
	selecao=new int[2];
	global=true;
	transformacoes=new vector<Transformacao *>();
	curvthresholds=new vector<double *>();
	esquemacor=1;
	changed=false;
	binsize=0;
	maxerror=0.04;
	disksize=0.028;
	drawdisks=true;
	iteracao=-1;
}

int mywindow::handle(int event)
{
   switch (event) {
			case FL_PUSH:
				push_event(Fl::event_x(),Fl::event_y(),Fl::event_button3());
				break;
            case FL_RELEASE:
				release_event(Fl::event_x(),Fl::event_y());
				break;
            case FL_DRAG:
				drag_event(Fl::event_x(),Fl::event_y(),Fl::event_button1(),Fl::event_button3());
				break;
			case FL_MOUSEWHEEL:
				wheel_event(Fl::event_dy());
				break;
            default:
                return Fl_Window::handle(event);
        }
}


void mywindow::push_event(int x, int y, int middle)
{
	ultimomouse=surfaceProject(x,y);
}

Vec3 mywindow::surfaceProject(int x, int y)
{
	Vec3 temp=Vec3((double)(2*x-w())/(double)w(),(double)(h()-2*y)/(double)h(),0);
	double tempnorm;
	if((tempnorm=norm(temp))>1.0)
		tempnorm=1.0;
	temp(2)=sqrt(1-tempnorm*tempnorm);
	unitize(temp);
	return temp;
}

void mywindow::drag_event(int x, int y, int b1, int b3)
{
	Vec3 p1=surfaceProject(x,y);
	Vec3 rotaxis=ultimomouse^p1;
	double angle=acos((p1/norm(p1))*(ultimomouse/norm(ultimomouse)));
	glLoadIdentity();
	glRotatef(angle*180/M_PI,rotaxis(0),rotaxis(1),rotaxis(2));
	glMultMatrixf( (GLfloat *) modelmat );
	glGetFloatv(GL_MODELVIEW_MATRIX,(GLfloat *) modelmat);
	ultimomouse=p1;
	changed=true;
	redraw();
}

void mywindow::wheel_event(int dy)
{
	scroll+=vel*dy;
	changed=true;
	redraw();
}

void mywindow::release_event(int x, int y)
{	
	ultimomouse=surfaceProject(x,y);
}

void mywindow::setup()
{
	glMatrixMode(GL_PROJECTION);    
	glLoadIdentity();
	gluPerspective(45.f,(float)this->w()/(float)this->h(),.1f,10.0f);
	glMatrixMode(GL_MODELVIEW);		
}


void mywindow::desenharSuperficies()
{

	if(desenharReconstrucao)
		if(iteracoes->size()>iteracao && iteracao>=0)
		{
			glPushMatrix();
			glLoadIdentity();
			gluLookAt(0,0,5-scroll,0,0,0,0,1,0);
			glMultMatrixf( (GLfloat *) modelmat );
			float cor[4]={0.8,0.8,0.8,1.};
			glEnable(GL_LIGHTING);
			glMaterialfv(GL_FRONT,GL_DIFFUSE,cor);
			glMaterialfv(GL_FRONT,GL_SPECULAR,cor);
			glColor3f(0,0,1);
			Vec3 p1(0,0,1);
			for(int i=0;i<iteracoes->at(iteracao)->rec_strat->size();i++)
			{
				if(iteracoes->at(iteracao)->rec_strat->at(i)->v->erro>-0.5)
				{
					Vec3 normalit=Vec3(iteracoes->at(iteracao)->rec_strat->at(i)->v->n.g(0),iteracoes->at(iteracao)->rec_strat->at(i)->v->n.g(1),iteracoes->at(iteracao)->rec_strat->at(i)->v->n.g(2));
					Vec3 rotaxis=p1^normalit;
					unitize(rotaxis);
					cor[0]=0.8+0.2*iteracoes->at(iteracao)->rec_strat->at(i)->v->erro/maxerror;
					cor[1]=0.8-0.8*iteracoes->at(iteracao)->rec_strat->at(i)->v->erro/maxerror;
					cor[2]=0.8-0.8*iteracoes->at(iteracao)->rec_strat->at(i)->v->erro/maxerror;
					glMaterialfv(GL_FRONT,GL_DIFFUSE,cor);
					glMaterialfv(GL_FRONT,GL_SPECULAR,cor);
					glPushMatrix();
					glTranslatef(iteracoes->at(iteracao)->rec_strat->at(i)->v->a.g(0),iteracoes->at(iteracao)->rec_strat->at(i)->v->a.g(1),iteracoes->at(iteracao)->rec_strat->at(i)->v->a.g(2));
					glRotatef(acos((p1)*(normalit/norm(normalit)))*180/M_PI,rotaxis(0),rotaxis(1),rotaxis(2));
					GLUquadricObj *qObj=gluNewQuadric();							
					if(drawdisks)
						gluDisk(qObj,0,disksize,10,2);
					else
						gluSphere(qObj,disksize,6,6);
					gluDeleteQuadric(qObj);
					glPopMatrix();
				}
			}
			glPopMatrix();
		}

	if(desenharMalhas)
		if(iteracoes->size()>iteracao && iteracao>=0)
		{
			glPushMatrix();
			glLoadIdentity();
			gluLookAt(0,0,5-scroll,0,0,0,0,1,0);
			glMultMatrixf( (GLfloat *) modelmat );

			for(int nummalha=0;nummalha<iteracoes->at(iteracao)->stratifications->size();nummalha++)
			{			

				float cor[4];
				if(esquemacor==1)
				{
					cor[0]=tabcor[nummalha%13][0];
					cor[1]=tabcor[nummalha%13][1];
					cor[2]=tabcor[nummalha%13][2];
					cor[3]=tabcor[nummalha%13][3];
				}
				glMaterialfv(GL_FRONT,GL_DIFFUSE,cor);
				glMaterialfv(GL_FRONT,GL_SPECULAR,cor);
				Vec3 p1(0,0,1);
				for(int k=0;k<iteracoes->at(iteracao)->stratifications->at(nummalha)->size();k++)
				{
					Vec3 normalit=Vec3(iteracoes->at(iteracao)->stratifications->at(nummalha)->at(k)->n.g(0),iteracoes->at(iteracao)->stratifications->at(nummalha)->at(k)->n.g(1),iteracoes->at(iteracao)->stratifications->at(nummalha)->at(k)->n.g(2));
					Vec3 rotaxis=p1^normalit;
					unitize(rotaxis);
					glPushMatrix();
					glTranslatef(iteracoes->at(iteracao)->stratifications->at(nummalha)->at(k)->a.g(0),iteracoes->at(iteracao)->stratifications->at(nummalha)->at(k)->a.g(1),iteracoes->at(iteracao)->stratifications->at(nummalha)->at(k)->a.g(2));
					glRotatef(acos((p1)*(normalit/norm(normalit)))*180/M_PI,rotaxis(0),rotaxis(1),rotaxis(2));
					GLUquadricObj *qObj=gluNewQuadric();							
					if(drawdisks)
						gluDisk(qObj,0,disksize,10,2);
					else
						gluSphere(qObj,disksize,6,6);
					gluDeleteQuadric(qObj);
					glPopMatrix();
				}

			}
			glPopMatrix();
		}
}

void mywindow::draw()
{
	if(!valid())
	{
		glClearDepth(1.0f);							
		glEnable(GL_DEPTH_TEST);						
		glDepthFunc(GL_LEQUAL);
		glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
		glShadeModel(GL_SMOOTH);
		
		glClearColor(1,1,1,0);	
		glClearDepth(1.0f);
		
		setup();
		glLoadIdentity();
		glGetFloatv(GL_MODELVIEW_MATRIX,(GLfloat *) modelmat);
		float diffuselight[4]={1,1,1,1};
		float headposition[4]={0,0,1,0};
		glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuselight);
		glLightfv(GL_LIGHT1, GL_POSITION, headposition);
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
		glEnable(GL_LIGHTING);	
		glEnable(GL_LIGHT1);
		glMateriali(GL_FRONT,GL_SHININESS,120);
	}
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	desenharSuperficies();
}


mywindow::~mywindow(void)
{
}