#pragma once
#include <FL/Fl.h>
#include <FL/Gl.h>
#include <FL/Fl_Gl_Window.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Double_Window.H>
#include <GL/glu.h>
#include "math.h"
#include <vector>

#include <stdio.h>
#include <queue>
#include <deque>
#include "defs.h"
#include <time.h>
#include "structs.h"

using namespace std;

typedef deque<double, allocator<double> > DBLDQU;
typedef priority_queue<double,DBLDQU, greater<double> > DBLPRQUE;

typedef priority_queue<double,DBLDQU, less<double> > DBLPRQUE2;



struct Transformacao{
	double angx;
	double angy;
	double transx;
	double transy;
};

class mywindow:
	public Fl_Gl_Window
{
private:
	void draw();	
	int *selecao;
	void setup();
	void desenharSuperficies();
	Vec3 baricentro;
	int qmalhas;
	int handle_mouse(int event, int button, int x, int y);
	void push_event(int x, int y, int middle);
	void release_event(int x, int y);
	void drag_event(int x, int y, int b1, int b3);
	void wheel_event(int dy);
	bool changed;
	clock_t starttime;
	int screenshotcount;
	float modelmat[16];
	float capmat[16];
	Vec3 surfaceProject(int x, int y);
	bool primeirasi;
	double binsize;


public:
	Vec3 eixoX,eixoY,eixoZ;
	bool drawdisks;
	float maxerror;
	float disksize;
	vector<Iteration *> *iteracoes;
	int iteracao;
	vector<Mesh *> *malhas;
	Mesh *reconstructed;
	bool capbutpush;
	bool global;
	vector<double *> *curvthresholds;
	int  esquemacor;
	vector<Transformacao *> *transformacoes;
	float scroll;
	float vel;
	Vec3 ultimomouse;
	Vec3 pushmouse;
	bool desenharReconstrucao;
	bool desenharMalhas;
	bool desenharPontos;
	bool iluminacaoPorVertice;
	void calcularThresholds();
	mywindow(int X, int Y, int W, int H, const char *L);
	//void calcularCentro();
	int handle(int event);
	virtual ~mywindow(void);
};

static const float tabcor[13][4] =
{ 
	0.0, 0.8, 0.0, 1.0,
	0.0, 0.0, 0.8, 1.0,
	1.0, 0.5, 0.5, 1.0,
	0.5, 1.0, 0.5, 1.0,
	0.5, 0.5, 1.0, 1.0,
	1.0, 1.0, 0.0, 1.0,
	0.0, 1.0, 1.0, 1.0,
	1.0, 0.0, 1.0, 1.0,
	0.5, 0.9, 0.0, 1.0,
	1.0, 1.0, 0.5, 1.0,
	0.5, 1.0, 1.0, 1.0,
	1.0, 0.5, 1.0, 1.0,
	0.8, 0.0, 0.0, 1.0
};
